/*
								+---------------------------------+
								|                                 |
								|  ***   Lame common dlgs   ***  |
								|                                 |
								|  Copyright   -tHE SWINe- 2005  |
								|                                 |
								|           Dialogs.cpp           |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 11-13-2006
 *
 *	all integers, containing just true or false were replaced by bool
 *	added bool CLoggerDialog::AddLineF(const char *p_s_format_str, ...) function
 *	added function-documenting comments (todo)
 *	note it's possible to use GetModuleHandle(NULL) in DLL mode
 *
 */

#include <vector>
#include "CallStack.h"
#include <windows.h>
#include "DialogsRes.h"
#include "Dialogs.h"

#ifdef __UBERLAME_DLL
static HINSTANCE h_instance; // note it's possible to use GetModuleHandle(NULL)

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	h_instance = hinstDLL;

	return TRUE;
}
// dll-main to obtain dll instance (necessary to access resources)
#endif //__UBERLAME_DLL

/*
 *								=== CLameDialog ===
 */

std::vector<CLameDialog*> *CLameDialog::m_p_dialog_list = 0;
CLameDialog *CLameDialog::m_p_recent_dialog;
CLameDialog *CLameDialog::m_p_dialog_being_created = 0;

CLameDialog::CLameDialog(int n_resource_id)
	:m_b_being_destroyed(false),
	m_n_resource_id(n_resource_id),
	m_h_dlg((HWND)0/*INVALID_HANDLE_VALUE*/)
{
	if(!m_p_dialog_list)
		m_p_dialog_list = new std::vector<CLameDialog*>;
	m_p_dialog_list->push_back(this);
	_ASSERTE(m_p_dialog_list->size());
	m_p_recent_dialog = this;
}

CLameDialog::~CLameDialog()
{
	if(m_h_dlg != 0/*INVALID_HANDLE_VALUE*/) {
		End();
	}
	for(int i = 0; i < m_p_dialog_list->size(); i ++) {
		if((*m_p_dialog_list)[i] == this) {
			m_p_dialog_list->erase(&(*m_p_dialog_list)[i]);
			if(!m_p_dialog_list->size()) {
				delete m_p_dialog_list;
				m_p_dialog_list = 0;
			}
			return;
		}
	}
	_ASSERTE(0); // it wasn't in the list
}

bool CLameDialog::Create(HWND h_parent, HINSTANCE h_instance)
{
	m_b_being_destroyed = false;
	m_p_dialog_being_created = this;
	// there are some messages sent before h_dlg is returned so we need
	// to know which dialog object we should link them to ...

	if((m_h_dlg = CreateDialog(h_instance, MAKEINTRESOURCE(m_n_resource_id),
	   h_parent, GlobalDlgProc)) == 0/*INVALID_HANDLE_VALUE*/) {
		int e = GetLastError();
		m_p_dialog_being_created = 0;
		return false;
	}
	m_p_dialog_being_created = 0;
	return true;
}

bool CLameDialog::End()
{
	m_b_being_destroyed = true;
	EndDialog(m_h_dlg, 0);
	m_h_dlg = (HWND)0/*INVALID_HANDLE_VALUE*/;

	return true;
}

bool CLameDialog::b_Created()
{
	return m_h_dlg != 0/*INVALID_HANDLE_VALUE*/;
}

int CALLBACK CLameDialog::GlobalDlgProc(HWND h_dlg, unsigned int n_msg,
	unsigned int n_w_param, long n_l_param)
{
	if(!m_p_recent_dialog || m_p_recent_dialog->m_h_dlg != h_dlg) {
		m_p_recent_dialog = 0;
		for(int i = 0; i < m_p_dialog_list->size(); i ++) {
			if((*m_p_dialog_list)[i]->m_h_dlg == h_dlg) {
				m_p_recent_dialog = (*m_p_dialog_list)[i];
				break;
			}
		}
	}
	// find appropriate window class ...

	if(!m_p_recent_dialog) {
		if(m_p_dialog_being_created) {
			m_p_recent_dialog = m_p_dialog_being_created;
			m_p_dialog_being_created->m_h_dlg = h_dlg;
		} else {
			//_ASSERTE(0);
			return 0;
		}
	}
	// may happen when we end dialog with some messages in queue

	if(m_p_recent_dialog->m_b_being_destroyed)
		return 0;
	return m_p_recent_dialog->DlgProc(h_dlg, n_msg, n_w_param, n_l_param);
}

/*
 *								=== ~CLameDialog ===
 */

/*
 *								=== CLoggerDialog ===
 */

int CLoggerDialog::DlgProc(HWND h_dlg, unsigned int n_msg,
	unsigned int n_w_param, long n_l_param)
{
	switch(n_msg) {
	case WM_INITDIALOG:
		return true;

	case WM_CLOSE:
		EndDialog(h_dlg, 0);
		h_dlg = 0;
		return 0;

	case WM_COMMAND:
		if(n_w_param == IDCANCEL) {
			EndDialog(h_dlg, 0);
			h_dlg = 0;
		}
		return 0;
	}

	return 0;
}

bool CLoggerDialog::AddLine(const char *p_s_str)
{
	if(!b_Created())
		return false;

	HWND h_listbox = GetDlgItem(m_h_dlg, IDC_LOG_LIST);

	if(SendMessage(h_listbox, LB_GETCOUNT, 0, 0) > m_n_max_lines - 1)
		SendMessage(h_listbox, LB_DELETESTRING, 0, 0);
	// erase first line if number of lines reaches maximum

	if(SendMessage(h_listbox, LB_ADDSTRING, 0, (int)p_s_str) <= 0)
		return false;
	// add string

	SendMessage(h_listbox, LB_SETCURSEL, SendMessage(h_listbox, LB_GETCOUNT, 0, 0) - 1, 0);
	// scroll down

	MSG t_msg;
	while(PeekMessage(&t_msg, m_h_dlg, 0, 0, PM_REMOVE)) { 
        TranslateMessage(&t_msg);
        DispatchMessage(&t_msg);
    }
	// update dialog so we can see changes reflected

	return true;
}

bool CLoggerDialog::AddLineF(const char *p_s_format_str, ...)
{
	va_list ap;
	int n_size = strlen(p_s_format_str) + 1;
	char *p_s_str;

	if(!(p_s_str = new char[n_size]))
		return false;
	// alloc str as long as formatting str (should be enough in some cases)

	while(1) {
		va_start(ap, p_s_format_str);
#if defined(_MSC_VER) && !defined(__MWERKS__) // msvc
		int n = _vsnprintf(p_s_str, n_size - 1, p_s_format_str, ap); // maximum characters to write, not the size of buffer
#else
		int n = vsnprintf(p_s_str, n_size, p_s_format_str, ap);
#endif
		va_end(ap);
		// try to sprintf

		if(n > -1 && n < n_size) {
#if defined(_MSC_VER) && !defined(__MWERKS__) // msvc
			p_s_str[n] = 0;
#endif
			return AddLine(p_s_str);
		}
		// see if we made it

		if(n > -1) // glibc 2.1
			n_size = n + 1; // precisely what is needed
		else // glibc 2.0, msvc
			//n_size *= 2; // twice as much ... can be wasteful sometimes
			n_size += 16; // decided to be better sollution

		delete[] p_s_str;
		if(!(p_s_str = new char[n_size]))
			return false;
		// realloc to try again
	}
}

/*
 *								=== ~CLoggerDialog ===
 */

/*
 *								=== CRenderPathDialog ===
 */

#include <commctrl.h>

bool CRenderPathDialog::AddPath(const char *p_s_name, int n_supported_materials, int n_materials)
{
	if(!b_Created())
		return false;

	HWND h_listbox = GetDlgItem(m_h_dlg, IDC_PATH_LIST);
	/*if(SendMessage(h_listbox, LB_ADDSTRING, 0, (int)p_s_name) < 0)
		return false;*/

	LVITEM t_item;

	t_item.mask = LVIF_TEXT;
	t_item.iImage = -1;
	t_item.iItem = SendMessage(h_listbox, LVM_GETITEMCOUNT, 0, 0);
	t_item.iSubItem = 0;

	char p_s_buffer[256] = {0};
	strncpy(p_s_buffer, p_s_name, 255);
	t_item.pszText = p_s_buffer;
	SendMessage(h_listbox, LVM_INSERTITEM, 0, (int)&t_item); // insert item

	sprintf(p_s_buffer, "%d / %d", n_supported_materials, n_materials);
	t_item.iSubItem = 1;
	t_item.pszText = p_s_buffer;
	SendMessage(h_listbox, LVM_SETITEM, 0, (int)&t_item); // insert item

	return true;
}

bool CRenderPathDialog::SelectPath(const char *p_s_name)
{
	if(!b_Created())
		return false;

	HWND h_listbox = GetDlgItem(m_h_dlg, IDC_PATH_LIST);
	for(int i = 0, n_count = SendMessage(h_listbox, LVM_GETITEMCOUNT, 0, 0); i < n_count; i ++) {
		LVITEM t_item = {0};
		char p_s_buffer[256] = {0};

		t_item.iSubItem = 0;
		t_item.iItem = i;
		t_item.cchTextMax = 256;
		t_item.pszText = p_s_buffer;

		SendMessage(h_listbox, LVM_GETITEMTEXT, i, (int)&t_item);

		if(!stricmp(p_s_buffer, p_s_name)) {
			SendMessage(h_listbox, LVM_SETSELECTIONMARK, 0, i);
			// select item ...

			//SendMessage(h_listbox, LVM_SETHOTITEM, i, 0);
			// set hot item (different color of text and hover)

			SendMessage(h_listbox, LVM_ENSUREVISIBLE, i, false);
			RECT t_rect, t_box_rect;
			t_rect.left = LVIR_BOUNDS;
			SendMessage(h_listbox, LVM_GETITEMRECT, i, (int)&t_rect);
			GetClientRect(h_listbox, &t_box_rect);
			t_rect.bottom += t_box_rect.top;
			t_rect.top += t_box_rect.top;
			t_rect.left += t_box_rect.left;
			t_rect.right += t_box_rect.left;
			SendMessage(h_listbox, WM_LBUTTONDOWN, 0,
				(t_rect.left + t_rect.right) / 2 + ((t_rect.bottom + t_rect.top) << 15));
			// a bit of hack ... send mousedown message with position right above
			// the item i want selected; i wasn't able to make that stupit listbox
			// to show the hover

			return true;
		}
	}
	// find item with string, equal to path name and select it

	return false;
}

bool CRenderPathDialog::ShaderLang(int n_lang_switch, bool b_arb_support, bool b_glsl_support)
{
	if(!b_Created())
		return false;

	if(n_lang_switch == 0)
		SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_DEFAULT), BM_SETCHECK, BST_CHECKED, 0);
	else if(n_lang_switch == 1)
		SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_GLSL), BM_SETCHECK, BST_CHECKED, 0);
	else
		SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_ARB), BM_SETCHECK, BST_CHECKED, 0);
	// set checks

	if(!b_glsl_support) {
	   EnableWindow(GetDlgItem(m_h_dlg, IDC_SHADER_GLSL), false);
	   EnableWindow(GetDlgItem(m_h_dlg, IDC_SHADER_DEFAULT), false); // force shaders to use the other available language
	   if(b_arb_support)
		   SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_ARB), BM_SETCHECK, BST_CHECKED, 0);
	}
	if(!b_arb_support) {
	   EnableWindow(GetDlgItem(m_h_dlg, IDC_SHADER_ARB), false);
	   EnableWindow(GetDlgItem(m_h_dlg, IDC_SHADER_DEFAULT), false); // force shaders to use the other available language
	   if(b_glsl_support)
		   SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_GLSL), BM_SETCHECK, BST_CHECKED, 0);
	}
	// disable if shading language(s) aren't supported

	return true;
}

bool CRenderPathDialog::CompressTextures(bool b_compress_textures)
{
	if(!b_Created())
		return false;

	SendMessage(GetDlgItem(m_h_dlg, IDC_COMPRESS_TEXTURES), BM_SETCHECK,
		(b_compress_textures)? BST_CHECKED : BST_UNCHECKED, 0);

	return true;
}

bool CRenderPathDialog::Create(HWND h_parent, HINSTANCE h_instance)
{
	InitCommonControls();
	// comctl32.dll ::InitCommonControls() - possibly better to link to dll dynamically

	if(!CLameDialog::Create(h_parent, h_instance))
		return false;

	EnableWindow(GetDlgItem(m_h_dlg, IDC_SET_DEFAULT_PATH), false);
	// enable "always use selected path" checkbox since we ignore it's state right now
	// todo

	HWND h_listbox = GetDlgItem(m_h_dlg, IDC_PATH_LIST);

	LVCOLUMN t_column;

	t_column.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	t_column.fmt = LVCFMT_LEFT;
	//
	t_column.pszText = "path name";
	t_column.cx = 75;
	SendMessage(h_listbox, LVM_INSERTCOLUMN, 0, (int)&t_column);
	//
	t_column.pszText = "supported mats";
	t_column.cx = 95;
 	SendMessage(h_listbox, LVM_INSERTCOLUMN, 1, (int)&t_column);
	
	SendMessage(h_listbox, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_FULLROWSELECT);
	::SetFocus(h_listbox);

	return true;
}

bool CRenderPathDialog::DoModal(int &r_n_chosen_path,
	bool &r_b_set_default, int &r_n_lang_switch, bool &r_b_compress_textures)
{
	if(!b_Created())
		return false;

	m_b_clicked_ok = false;

	MSG t_msg;
	while(GetMessage(&t_msg, m_h_dlg, 0, 0)) {
		TranslateMessage(&t_msg);
		DispatchMessage(&t_msg);

		if(m_b_clicked_ok) {
			HWND h_listbox = GetDlgItem(m_h_dlg, IDC_PATH_LIST);

			if((r_n_chosen_path = SendMessage(h_listbox, LVM_GETSELECTIONMARK, 0, 0)) == -1) {
				MessageBox(m_h_dlg, "You have to select path first", "Pick rendering path", MB_OK | MB_ICONINFORMATION);
				m_b_clicked_ok = false;
				continue;
			}
			// did we chose any path at all ?

			r_b_set_default = (SendMessage(GetDlgItem(m_h_dlg,
				IDC_SET_DEFAULT_PATH), BM_GETCHECK, 0, 0) == BST_CHECKED);
			// is "always use this path" checked ?

			r_n_lang_switch = (SendMessage(GetDlgItem(m_h_dlg,
				IDC_SHADER_GLSL), BM_GETCHECK, 0, 0) == BST_CHECKED) +
				(SendMessage(GetDlgItem(m_h_dlg, IDC_SHADER_ARB),
				BM_GETCHECK, 0, 0) == BST_CHECKED) * 2;
			// get lang switch status

			r_b_compress_textures = (SendMessage(GetDlgItem(m_h_dlg,
				IDC_COMPRESS_TEXTURES), BM_GETCHECK, 0, 0) == BST_CHECKED);
			// compress textures

			EndDialog(m_h_dlg, 0);

			m_b_clicked_ok = false;
		}

		if(!t_msg.message)
			break;
	}
	// message loop

	m_h_dlg = 0;

	return true;
}

int CRenderPathDialog::DlgProc(HWND h_dlg,
	unsigned int n_msg, unsigned int n_w_param, long n_l_param)
{
	switch(n_msg) {
	case WM_INITDIALOG:
		return true;

	case WM_KEYDOWN:
		if(n_w_param == VK_RETURN)
			m_b_clicked_ok = true;
		return 0;

	case WM_COMMAND:
		if(n_w_param == IDOK)
			m_b_clicked_ok = true;
		return 0;
	}

	return 0;
}

/*
 *								=== ~CRenderPathDialog ===
 */
